'use strict';

const _ = require('lodash');

class ObjectUtils {

    queryGet(obj, query) {
        query = query.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
        query = query.replace(/^\./, '');           // strip a leading dot

        let a = query.split('.');
        for(let i = 0, n = a.length; i < n; ++i) {
            let k = a[i];
            if(k in obj) {
                obj = obj[k];
            } else {
                return;
            }
        }
        return obj;
    }

    querySet(obj, key, value) {
        key = key.replace(/_/g, '.');
        key = key.replace(/\[(\w+)\]/g, '.$1');
        key = key.replace(/^\./, '');

        let parts = key.split('.');

        let lastKey = parts.pop();

        for(let i = 0, n = parts.length; i < n; ++i) {
            let k = parts[i];

            if(k in obj) {
                obj = obj[k];

            } else {
                obj[k] = {};
                obj = obj[k];
            }
        }

        obj[lastKey] = value;

        return obj;
    }

    simplify(object) {
        let transformResultNode = function(node) {
            if(node) {
                if(typeof node == 'object' && node.constructor && node.constructor.name == 'Key') {
                    node = node.id;

                } else if(typeof node.toArray != 'undefined') {
                    node = node.toArray();

                } else if(typeof node.toObject != 'undefined') {
                    node = node.toObject();

                } else if(typeof node.getValues != 'undefined') {
                    node = node.getValues();

                } else if(node && node.dataValues && node._previousDataValues) {
                    node = node.dataValues; // sequelize instance
                }
            }

            return node;
        };

        let traverseData = node => {
            node = transformResultNode(node);

            if(_.isPlainObject(node)) {
                node = _.mapValues(node, traverseData);

            } else if(_.isArray(node)) {
                node = _.map(node, traverseData);
            }

            return node;
        };

        return traverseData(object);
    }

}

module.exports = new ObjectUtils();