'use strict';

class ProcessUtils {

    extendConsoleOutput() {
        this.emitting = false;

        let events = [];

        console.on = function(name, cb) {
            events.push({
                name,
                cb,
            });
            return this;
        };

        let emit = (type, args) => {
            if(!this.emitting) {
                this.emitting = true;

                events.forEach(ev => {
                    if(ev.name === type) {
                        ev.cb.apply(console, args);
                    }
                });

                this.emitting = false;
            }
        };

        let consoleLog = console.log;
        let consoleInfo = console.info;
        let consoleWarn = console.warn;
        let consoleError = console.error;
        let consoleTrace = console.trace;
        let consoleAssert = console.assert;

        console.log = function() {
            emit('log', arguments);
            return consoleLog.apply(this, arguments);
        };

        console.info = function() {
            emit('info', arguments);
            return consoleInfo.apply(this, arguments);
        };

        console.warn = function() {
            emit('warn', arguments);
            return consoleWarn.apply(this, arguments);
        };

        console.error = function() {
            emit('error', arguments);
            return consoleError.apply(this, arguments);
        };

        console.trace = function() {
            emit('trace', arguments);
            return consoleTrace.apply(this, arguments);
        };

        console.assert = function() {
            emit('assert', arguments);
            return consoleAssert.apply(this, arguments);
        };

        return this;
    }

    extendProcessOutput() {
        this.emitting = false;

        let stdoutWrite = process.stdout.write;
        let stderrWrite = process.stderr.write;

        process.stdout.write = function() {
            if(!this.emitting) {
                this.emitting = true;
                this.emit('write', arguments);
                this.emitting = false;
            }

            return stdoutWrite.apply(process.stdout, arguments);
        };

        process.stderr.write = function() {
            if(!this.emitting) {
                this.emitting = true;
                this.emit('write', arguments);
                this.emitting = false;
            }

            return stderrWrite.apply(process.stderr, arguments);
        };

        return this;
    }

}

module.exports = new ProcessUtils;