'use strict';

const should = require('should');

const __ = require('../index');

describe('reflection', function() {
    this.timeout(10 * 1000);

    /* -------------------------------------------------------------------------
     * Before
     * ---------------------------------------------------------------------- */

    before(function() {
    });

    /* -------------------------------------------------------------------------
     * Create
     * ---------------------------------------------------------------------- */

    it('should extract method arguments', () => {
        should(__.reflection.getFunctionParams(function(a, b, c, d) {
        })).be.containDeepOrdered(['a', 'b', 'c', 'd']);

        should(__.reflection.getFunctionParams(function(a, /*b,c,*/d) {
        })).be.containDeepOrdered(['a', 'd']);

        should(__.reflection.getFunctionParams(function() {
        })).be.containDeepOrdered([]);

        should(__.reflection.getFunctionParams((a, b, c, d) => {
        })).be.containDeepOrdered(['a', 'b', 'c', 'd']);

        should(__.reflection.getFunctionParams((a, /*b,c,*/d) => {
        })).be.containDeepOrdered(['a', 'd']);

        should(__.reflection.getFunctionParams(() => {
        })).be.containDeepOrdered([]);
    });

    it('should invoke functions with mapped arguments', () => {
        let testFnc = (d, a, c, b) => {
            should(a).equal(1);
            should(b).equal(2);
            should(c).be.undefined();
            should(d).equal(4);

            return a + b + d;
        };

        let res = __.reflection.applyMapping(testFnc, this, {
            a: 1,
            b: 2,
            // c: 3,
            d: 4,
        });

        should(res).be.equal(7);
    });
});